﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Windows;
using Windows.Devices.Bluetooth;
using Windows.Devices.Bluetooth.Advertisement;

namespace WpfBleSampleApp
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        private BluetoothLEAdvertisementWatcher watcher;

        private Dictionary<string, BluetoothLEDevice> devices = new Dictionary<string, BluetoothLEDevice>();
        private Mutex mutex = new Mutex();

        private string[] okList =
        {
            "BluetoothLE#BluetoothLE98:5f:d3:d1:35:3f-74:99:95:70:e8:29",
        };

        public MainWindow()
        {
            InitializeComponent();
            this.watcher = new BluetoothLEAdvertisementWatcher();

            // CompanyIDとかDataでフィルタリングしたいとき
            //var md = new BluetoothLEManufacturerData();
            //// company id 0xFFFF (多分これ https://www.bluetooth.com/specifications/assigned-numbers/company-Identifiers)
            //md.CompanyId = 0xFFFF; 

            //// data 0x1234
            //var w = new DataWriter();
            //w.WriteUInt16(0x1234);
            //md.Data = w.DetachBuffer();
            //this.watcher.AdvertisementFilter.Advertisement.ManufacturerData.Add(md);

            // rssi >= -60のとき受信開始するっぽい
            this.watcher.SignalStrengthFilter.InRangeThresholdInDBm = -60;
            // rssi <= -65が2秒続いたら受信終わるっぽい
            this.watcher.SignalStrengthFilter.OutOfRangeThresholdInDBm = -65;
            this.watcher.SignalStrengthFilter.OutOfRangeTimeout = TimeSpan.FromMilliseconds(2000);
            this.watcher.Received += this.Watcher_Received;
        }

        private async void Watcher_Received(BluetoothLEAdvertisementWatcher sender, BluetoothLEAdvertisementReceivedEventArgs args)
        {
            /*
            await this.Dispatcher.InvokeAsync(() =>
            {
                var md = args.Advertisement.ManufacturerData.FirstOrDefault();
                if (md != null)
                {
                    // ManufactureDataをもとにCompanyIDとったりできる
                }
                var uuids = String.Join(",", args.Advertisement.ServiceUuids.Select(uuid => uuid.ToString()));
                this.TextBlockRSSI.Text = $"{args.Timestamp:HH\\:mm\\:ss}, RSSI: {args.RawSignalStrengthInDBm}, Address: {args.BluetoothAddress.ToString("X")}, Type: {args.AdvertisementType} , {args.Advertisement.ServiceUuids.Count}{uuids}";
            });
            */

            var task = BluetoothLEDevice.FromBluetoothAddressAsync(args.BluetoothAddress);
            //↑Adパケットをキャプチャするというタスク

            task.Completed = delegate
            {
                switch (task.Status)
                {
                    case Windows.Foundation.AsyncStatus.Completed://無事完了すれば
                        var device = task.GetResults();
                        if (device == null) return;
                        Debug.WriteLine($"device found: {device.Name}");

                        mutex.WaitOne();
                        devices[device.DeviceId] = device;

                        Debug.WriteLine("===========begin");
                        foreach (var x in devices)
                        {
                            var dev = x.Value;
                            var id = x.Key; // == dev.DeviceId

                            Debug.WriteLine($"{dev.Name}, {id}");
                        }
                        Debug.WriteLine("===========end");
                        mutex.ReleaseMutex();

                        this.Dispatcher.InvokeAsync(() =>
                        {
                            mutex.WaitOne();
                            this.TextBlockRSSI.Text = "";
                            foreach (var x in devices)
                            {
                                var dev = x.Value;
                                var id = x.Key; // == dev.DeviceId

                                this.TextBlockRSSI.Text += $"{dev.Name}, {id}, {dev.BluetoothAddress:x}\n";
                            }

                            // OKリストの中身と何個一致しているか
                            var count = okList.Count((okId) => devices.ContainsKey(okId));

                            this.TextBlockRSSI.Text += $"ok count: {count}";

                            mutex.ReleaseMutex();
                        });
                        break;

                    case Windows.Foundation.AsyncStatus.Started:
                        Debug.WriteLine($"started: ");
                        break;

                    default:
                        Debug.WriteLine($"failed: {task.Status} {task.ErrorCode}");
                        break;
                }
            };
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            this.watcher.Stop();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.watcher.Start();
        }
    }
}